// MainWindow.cpp: Implementierungsdatei
//

///////////////////////////////
// Copyright 2000 Andr Stein
// E-Mail: andre_stein@gmx.de
// WWW: http://stonemaster.port5.com
///////////////////////////////

#include "stdafx.h"
#include "OpenGLMfc.h"
#include "MainWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HALF_SCREEN_X 320
#define HALF_SCREEN_Y 240

#define INIT_X -7
#define INIT_Y 4.50

const float ZOOM_FACTOR = -15.0f;
const float GRENZE_X = 15.0/2.0f;
const float GRENZE_Y = 15.0/2.0f;

const float ambient[] = {0.25,0.25,0.25,1.0};
const float diffuse[] = {0.5,0.5,0.5,0.75};
const float specular[] = {0.75,0.75,0.75,1.0};
const float position[] = {0.0,0.0,5.0,1.0};



/////////////////////////////////////////////////////////////////////////////
// CMainWindow

CMainWindow::CMainWindow()
{
	mousePosition[0] = 0.0;
	mousePosition[1] = 0.0;
}

CMainWindow::~CMainWindow()
{
	delete clientDC;
}


BEGIN_MESSAGE_MAP(CMainWindow, CWnd)
	//{{AFX_MSG_MAP(CMainWindow)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_CLOSE()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CMainWindow 

int CMainWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	clientDC = new CClientDC(this);
	openGLDevice.create(clientDC->m_hDC);
	InitGL();
	
	return 0;
}

void CMainWindow::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	if (cy == 0)								
	{
		cy = 1;						
	}

	glViewport(0, 0, cx, cy);	

	glMatrixMode(GL_PROJECTION);						
	glLoadIdentity();						

	
	gluPerspective(45.0f,(GLfloat)cx/(GLfloat)cy,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);						
	glLoadIdentity();
}

BOOL CMainWindow::PreCreateWindow(CREATESTRUCT& cs) 
{
	screenMode.setDisplayMode(640,480,16);

	return CWnd::PreCreateWindow(cs);
}

void CMainWindow::Update()
{
	float frameend;
	CPoint mousePos;

	do
	{
		frameend = timer.getTime();
	}
	while (frameend == framestart);

	frametime = frameend - framestart;
	framestart = frameend;

	if (GetAsyncKeyState(VK_ESCAPE))
	{
		PostMessage(WM_CLOSE);
	}

	if (GetAsyncKeyState(VK_LBUTTON))
	{
		cards.click(mousePosition[0],mousePosition[1]);
	}

	GetCursorPos(&mousePos);
	SetCursorPos(HALF_SCREEN_X,HALF_SCREEN_Y);


	mousePosition[0] -= float((HALF_SCREEN_X - mousePos.x))/100.0f;
	mousePosition[1] += float((HALF_SCREEN_Y - mousePos.y))/100.0f;

	
	DrawFps();
	DrawGLScene();
	
	
	DrawMouse();

	SwapBuffers(clientDC->m_hDC);
}

void CMainWindow::DrawGLScene()
{
	glClear(GL_COLOR_BUFFER_BIT |
			GL_DEPTH_BUFFER_BIT);

	//glLoadIdentity();
	glPushMatrix();

	glTranslatef(0.0,0.0,ZOOM_FACTOR);
	

	glLightfv(GL_LIGHT0,GL_POSITION,position);

	glColor3f(1.0,1.0,1.0);
	cards.draw(frametime);
	
	
	

	glPopMatrix();

//	SwapBuffers(clientDC->m_hDC);
}

void CMainWindow::OnClose() 
{
	screenMode.reset();
	PostQuitMessage(0);

	CWnd::OnClose();
}

void CMainWindow::InitGL()
{
	glFrontFace(GL_CW);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);							
	glEnable(GL_DEPTH_TEST);					
	glDepthFunc(GL_LEQUAL);	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_CULL_FACE);

	glLightfv(GL_LIGHT0,GL_AMBIENT,ambient);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuse);
	glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0,GL_POSITION,position);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
	
	glMateriali(GL_FRONT,GL_SHININESS,128);
	glMaterialfv(GL_FRONT,GL_SPECULAR,specular);

	glEnable(GL_COLOR_MATERIAL);

	texture.createMipmap("Data/Back.jpg");
	cards.create(texture.getGLTexture(),INIT_X,INIT_Y);
	timer.create();
	font.createFont(clientDC->m_hDC,"Tahoma",40,0.0,SFontAttri(800,false,false));

	framestart = timer.getTime();
}

void CMainWindow::DrawMouse()
{
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	glBegin(GL_TRIANGLES);
		glColor4f(0.75,0.75,0.75,0.5);
		glVertex3f(mousePosition[0],mousePosition[1],ZOOM_FACTOR);
		glVertex3f(mousePosition[0]+0.5f,mousePosition[1],ZOOM_FACTOR);
		glVertex3f(mousePosition[0],mousePosition[1]-0.5f,ZOOM_FACTOR);
	glEnd();


	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}

void CMainWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Code fr die Behandlungsroutine fr Nachrichten hier einfgen und/oder Standard aufrufen
	switch (nChar)
	{
	case VK_SPACE:
		cards.create(texture.getGLTexture(),
			INIT_X,INIT_Y);
		break;
	}

	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMainWindow::DrawFps()
{
	static int frames = 0;
	static float fps = 0.0;

	frames += 1;

	if (frames > 100)
	{
		fps = float(frames)/frametime;
		frames = 0;
	}


//	glLoadIdentity();
	glPushMatrix();
	glDisable(GL_LIGHTING);

	glTranslatef(GRENZE_X-3.0,GRENZE_Y-3.0,ZOOM_FACTOR);
	glColor3f(1.0,1.0,1.0);
	//font.drawText(CENTER,"%0.0f",fps);

	glEnable(GL_LIGHTING);
	glPopMatrix();
}
