// SMemoryCard.cpp: Implementierung der Klasse SMemoryCard.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "SMemoryCard.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ROT_SPEED 0.2f
#define SET_ARRAY(a,n1,n2,n3) a[0] = n1; a[1] = n2; a[2] = n3;


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SMemoryCard::SMemoryCard()
{
		cardStatus = HIDDEN;
		position[0] = 0.0;
		position[1] = 0.0;
		backTexture = 0;
		cardType = 0;
		
}

SMemoryCard::~SMemoryCard()
{

}

void SMemoryCard::create(float xPos, float yPos, unsigned int back, int type)
{
	cardStatus = HIDDEN;
	rotation = 0.0;
	position[0] = xPos;
	position[1] = yPos;
	backTexture = back;
	cardType = type;
}

void SMemoryCard::draw(float frametime)
{
	glPushMatrix();
	glTranslatef(position[0],position[1],0.0);

	switch (cardStatus)
	{
	case HIDDEN:
		drawCard();
		break;
	case SHOWN:
		glRotatef(180.0,0.0,1.0,0.0);
		drawCard();
		break;
	case INVISIBLE:
		break;
	case TURNING_IN:
		rotation += ROT_SPEED*frametime;
		

		glRotatef(rotation,0.0,1.0,0.0);
		drawCard();
		
		if (rotation > 180.0)
		{
			rotation = 0.0;
			cardStatus = SHOWN;
		}
		break;
	case TURNING_OUT:
		rotation -= ROT_SPEED*frametime;
		

		glRotatef(rotation,0.0,1.0,0.0);
		drawCard();
		
		if (rotation < 0.0)
		{
			rotation = 0.0;
			cardStatus = HIDDEN;
		}
		break;
	}
		
	glPopMatrix();
}

int SMemoryCard::getType()
{
	return cardType;
}

SCardStatus SMemoryCard::getStatus()
{
	return cardStatus;
}

void SMemoryCard::drawCard()
{
	//////////
	//Vorne
	//////////
	if (cardStatus != SHOWN)
	{
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, backTexture);


		glBegin(GL_QUADS);
			glNormal3f(0.0f,0.0f,1.0f);
			glColor3f(1.0f,1.0f,1.0f);
			glTexCoord2f(0.0f,1.0f); glVertex3f(-xSize,ySize,0.0f);
			glTexCoord2f(1.0f,1.0f); glVertex3f(xSize,ySize,0.0f);
			glTexCoord2f(1.0f,0.0f); glVertex3f(xSize,-ySize,0.0f);
			glTexCoord2f(0.0f,0.0f); glVertex3f(-xSize,-ySize,0.0f);
		glEnd();

		glDisable(GL_TEXTURE_2D);
	}

	if (cardStatus != HIDDEN)
	{
				
		float* color = getCardColor();

		glBegin(GL_QUADS);
			glNormal3f(0.0,0.0,-1.0);
			glColor3fv(color);
			/*glTexCoord2f(0.0,1.0);*/ glVertex3f(xSize,ySize,-0.040f);
			/*glTexCoord2f(1.0,1.0);*/ glVertex3f(-xSize,ySize,-0.040f);
			/*glTexCoord2f(1.0,0.0);*/ glVertex3f(-xSize,-ySize,-0.040f);
			/*glTexCoord2f(0.0,0.0);*/ glVertex3f(xSize,-ySize,-0.040f);
		glEnd();

		delete color;
	}
}

void SMemoryCard::setStatus(SCardStatus state)
{
	if (state == TURNING_OUT)
	{
		rotation = 180.0;
	}

	cardStatus = state;
}

float* SMemoryCard::getCardColor()
{
	float* color = new float[3];

	switch (cardType)
	{
	case 1:
		SET_ARRAY(color,1.0f,0.0f,0.0f)//Rot
		break;
	case 2:
		SET_ARRAY(color,0.0f,1.0f,0.0f)//Gruen
		break;
	case 3:
		SET_ARRAY(color,0.0f,0.0f,1.0f)//Blau
		break;
	case 4:
		SET_ARRAY(color,0.0f,1.0f,1.0f)//Hellblau
		break;
	case 5:
		SET_ARRAY(color,1.0f,1.0f,1.0f)//Weiss
		break;
	case 6:
		SET_ARRAY(color,1.0f,0.0f,1.0f)//Rosa
		break;
	case 7:
		SET_ARRAY(color,0.5f,0.5f,0.5f)//Grau
		break;
	case 8:
		SET_ARRAY(color,0.5f,0.75f,0.5f)//Hellgruen
		break;
	case 9:
		SET_ARRAY(color,1.0f,1.0f,0.0f)//Gelb
		break;
	case 10:
		SET_ARRAY(color,0.5f,0.5f,0.0f)
		break;
	case 11:
		SET_ARRAY(color,0.0f,0.25f,0.25f)
		break;
	case 12:
		SET_ARRAY(color,0.4f,0.5f,0.6f)
		break;
	default:
		SET_ARRAY(color,0.0f,0.0f,0.0f)
	}

	return color;
}

bool SMemoryCard::isInside(float xPos, float yPos)
{
	float realX = position[0];
	float realY = position[1];

	bool xTrue = (xPos >= realX-xSize && xPos <= realX+xSize);
	bool yTrue = (yPos >= realY-ySize && yPos <= realY+ySize);

	return (xTrue && yTrue);
}

void SMemoryCard::click()
{
	if (cardStatus == HIDDEN)
	{
		setStatus(TURNING_IN);
	}
	else if (cardStatus == SHOWN)
	{
		setStatus(TURNING_OUT);
	}
}
