// STexture.cpp: Implementierung der Klasse STexture.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://stonemaster.port5.com
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "STexture.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

STexture::STexture()
{
	texture = 0;
	hBitmap = NULL;
}

STexture::~STexture()
{

}

STexture::STexture(const char* filename,int minFilter,int magFilter)
{
	createTexture(filename,minFilter,magFilter);	
}


///////////////////////////////
// Funktionen
///////////////////////////////

bool STexture::createTexture(const char *filename, int minFilter, int magFilter)
{
	BITMAP textureBitmap;
	bool noError;

	noError = loadBitmap(filename,textureBitmap);

	if (noError)
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		glTexImage2D(GL_TEXTURE_2D,0,3,textureBitmap.bmWidth,
			textureBitmap.bmHeight,0,GL_BGR_EXT,
			GL_UNSIGNED_BYTE,textureBitmap.bmBits);
		
		DeleteObject(hBitmap);
		
		return true;
	}

	
	
	return false;
}

bool STexture::createMipmap(const char *filename, int minFilter, int magFilter)
{
	BITMAP textureBitmap;
	bool noError;

	noError = loadBitmap(filename,textureBitmap);

	if (noError)
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		gluBuild2DMipmaps(GL_TEXTURE_2D,3,textureBitmap.bmWidth,
			textureBitmap.bmHeight,GL_BGR_EXT,
			GL_UNSIGNED_BYTE,textureBitmap.bmBits);

		DeleteObject(hBitmap);

		return true;
	}
	
	return false;
}

GLuint STexture::getGLTexture()
{
	return texture;
}

bool STexture::loadBitmap(const char *filename,BITMAP& bitmap)
{
	hBitmap = (HBITMAP)LoadImage(NULL,filename,IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION |
		LR_LOADFROMFILE);

	if (hBitmap)
	{
		GetObject(hBitmap,sizeof(bitmap),&bitmap);

		return true;
	}

	return false;
}

void STexture::select()
{
	glBindTexture(GL_TEXTURE_2D,texture);
}
